require 'uu/os/logger/log_level'
require 'uu/os/logger/log4r_plugin'
require 'uu/os/logger/mdc_provider'

module UU
  module OS

    # Logger component.
    class Logger
      
      # Logger name.
      # @return [String]
      attr_reader :name
      
      # Creates a new instance of Logger.
      #
      # @param [String, Class] name Logger name. If it is a String, it is used as it is.
      #            Otherwise name of its class is used.
      # @param [Hash] options Additional logger options. Parameter is optional, actual
      #             options depend on particular logger/outputter implementation and
      #             are to be found in corresponding documentation.
      def initialize(name, options = {})
        raise ArgumentError, 'Logger name is required' unless name
        
        if (name.kind_of?String) && !name.empty?
          logger_name = name
        elsif (name.kind_of?Class)
          logger_name = name.name
        elsif (name.kind_of?Module)
          logger_name = name.name
        else
          logger_name = name.class.name
        end
        @name = logger_name
        
        @plugin = UU::OS::Logger::Log4rPlugin.new(self.name, options)
      end
      
      # Logs error message.
      #
      # @param [String] msg Message
      # @param [Exception] exception Exception to be logged (optional)
      # @param [Array<String>] params Message parameters (optional)
      def error(msg, exception=nil, *params)
        if error_loggable?
          self.plugin.error(msg, exception, *params)
        end
      end
      
      # Checks if error messages can be logged.
      def error_loggable?
        return self.plugin.error_loggable?
      end
      
      # Logs warning message.
      #
      # @param [String] msg Message
      # @param [Exception] exception Exception to be logged (optional)
      # @param [Array<String>] params Message parameters (optional)
      def warning(msg, exception=nil, *params)
        if warning_loggable?
          self.plugin.warning(msg, exception, *params)
        end
      end
      
      # Checks if warning messages are enabled.
      def warning_loggable?
        return self.plugin.warning_loggable?
      end
      
      # Logs info message.
      #
      # @param [String] msg Message
      # @param [Exception] exception Exception to be logged (optional)
      # @param [Array<String>] params Message parameters (optional)
      def info(msg, exception=nil, *params)
        if info_loggable?
          self.plugin.info(msg, exception, *params)
        end
      end
      
      # Checks if info messages are enabled.
      def info_loggable?
        return self.plugin.info_loggable?
      end
      
      # Logs debug message.
      #
      # @param [String] msg Message
      # @param [Exception] exception Exception to be logged (optional)
      # @param [Array<String>] params Message parameters (optional)
      def debug(msg, exception=nil, *params)
        if debug_loggable?
          self.plugin.debug(msg, exception, *params)
        end
      end
      
      # Checks if debug messages are enabled.
      def debug_loggable?
        return self.plugin.debug_loggable?
      end
      
      # Logs message.
      #
      # @param [UU::OS::Logger::LogLevel] level Log level
      # @param [String] msg Message
      # @param [Exception] exception Exception to be logged (optional)
      # @param [Array<String>] params Message parameters (optional)
      def log(level, msg, exception=nil, *params)
        if loggable?(level)
          self.plugin.log(msg, exception, *params)
        end
      end
      
      # Checks if given log level is enabled.
      #
      # @param [UU::OS::Logger::LogLevel] level Log level
      def loggable?(level)
        return self.plugin.loggable?(level)
      end
      
      # Returns enabled logger level.
      def level
        return self.plugin.level
      end

      # Put a context value as identified by key into mapped diagnostic context.
      # Key and value can be nil.
      #
      # @param [String] key
      # @param [String] value
      def mdc_put(key, value)
        self.plugin.mdc_put(key, value)
      end

      # Remove the context identified by the key parameter from mapped diagnostic context.
      # Key can be nil.
      #
      # @param [String] key
      def mdc_remove(key)
        self.plugin.mdc_remove key
      end

      # @private
      def to_s
        "#{self.class.name} [name=#{self.name}, plugin=#{self.plugin.class.name}]"
      end

    protected
      
      attr_accessor :plugin
      attr_writer :name
      
    end
  end
end
