require 'uu/os/env/environment'
require 'uu/os/qos/qos_limit_exception'
require 'uu/os/logger'

module UU
  module OS
    module QoS

      # QoS handler component.
      module QoSHandler
        extend self

        # Relative path to configuration file
        # @private
        @@CFG_FILE_PATH = 'config/uu-client.properties'

        # Name of thread parameter for storing default maximum client timeout
        # @private
        @@MAX_TIMEOUT_PARAM = 'uu.os.qos.max-client-timeout'

        # Default maximum client timeout
        # @private
        @@DEFAULT_MAX_TIMEOUT = 300 # 5 minutes
        
        # Instance of logger.
        # @private
        @@log = UU::OS::Logger.new(self)

        # Returns default maximum client timeout (this timeout is used in case
        # no timeout is found in configuration file.
        #
        # @return [Numeric] Default maximum client timeout
        def self.default_max_client_timeout
          if !Thread.current[@@MAX_TIMEOUT_PARAM]
            QoSHandler.default_max_client_timeout=@@DEFAULT_MAX_TIMEOUT
          end
          Thread.current[@@MAX_TIMEOUT_PARAM]
        end

        # Stores default maximum client timeout (this timeout is used in case
        # no timeout is found in the configuration file.
        #
        # @param [Numeric] max_client_timeout Default maximum client timeout
        def self.default_max_client_timeout=(max_client_timeout)
          Thread.current[@@MAX_TIMEOUT_PARAM] = max_client_timeout
        end

        # Automatically wait for QoSHandler renewal.
        #
        # @param [Proc] block Block of code to execute
        def auto_retry(&block)
          # Load configuration uu.QoSHandler.max-client-timeout (in minutes)
          remainingTime = QoSHandler.default_max_client_timeout
          begin
            propt = Environment.load_configuration(@@CFG_FILE_PATH)
            remainingTime = propt[@@MAX_TIMEOUT_PARAM].to_i * 60 # seconds
          rescue
            # No problem, we will use default value
          end
          infiniteTimeout = (remainingTime < 0)
          while true do
            begin
              block.call
              break
            rescue QoSLimitException => e
              params = e.messages[0].params
              wait = (params[(params.size - 1)].to_i / 1000.0).to_i # value in exception is millis
              remainingTime -= wait
              if (!infiniteTimeout) && (remainingTime < 0)
                # Wait will exceed the maximum given time - throw QoSHandler limit exception
                raise e
              end
              @@log.warning('QoS limit exceeded. Waiting %s minutes for next attempt.', nil, wait / 60.0)
              sleep (wait)
            end
          end
        end

      end
    end
  end
end
