require 'uu/os/uesuri'

module UU
  module OS

    # Builder for creating UESURI.
    #
    # @example Usage:
    #          uri = UESURIBuilder.set_territory_id('123').set_artifact_code('artCode').set_object_id('456').to_uesuri
    class UESURIBuilder

      # Creates new instance of UESURIBuilder.
      def initialize
        @role_id = nil
        @role_code = nil
        @territory_id = nil
        @territory_code = nil
        @artifact_id = nil
        @artifact_code = nil
        @object_id = nil
        @object_code = nil
        @use_case_id = nil
        @use_case_code = nil
        @action = nil
        @parameters = nil
        @fragment = nil
      end

      # Parses specified string into UESURIBuilder instance.
      #
      # @param [String, UU::OS::UESURI] uesuri Parsed UESURI
      #
      # @return [UESURIBuilder] Instance of UESURIBuilder
      def self.parse_uesuri(uesuri)
        builder = UESURIBuilder.new
        builder.parse_uesuri(uesuri)
        return builder
      end

      # Parses specified string into UESURIBuilder instance.
      #
      # @param [String, UU::OS::UESURI] uesuri Parsed UESURI
      #
      # @return [UESURIBuilder] Instance of UESURIBuilder
      def parse_uesuri(uesuri)
        uri = UU::OS::UESURI.new(uesuri)
        @role_id = uri.role_id
        @role_code = uri.role_code
        @territory_id = uri.territory_id
        @territory_code = uri.territory_code
        @artifact_id = uri.artifact_id
        @artifact_code = uri.artifact_code
        @object_id = uri.object_id
        @object_code = uri.object_code
        @use_case_id = uri.use_case_id
        @use_case_code = uri.use_case_code
        @action = uri.action
        @parameters = uri.parameters
        @fragment = uri.fragment
        return self
      end

      # Sets role ID for the new UESURI.
      #
      # @param [String] role_id Role ID
      #
      # @return [UESURIBuilder] Instance of UESURIBuilder
      def self.set_role_id(role_id)
        builder = UESURIBuilder.new
        builder.set_role_id(role_id)
        return builder
      end

      # Sets role ID for the new UESURI.
      #
      # @param [String] role_id Role ID
      #
      # @return [UESURIBuilder] Instance of UESURIBuilder
      def set_role_id(role_id)
        @role_id = role_id
        return self
      end

      # Sets role code for the new UESURI.
      #
      # @param [String] role_code Role code
      #
      # @return [UESURIBuilder] Instance of UESURIBuilder
      def self.set_role_code(role_code)
        builder = UESURIBuilder.new
        builder.set_role_code(role_code)
        return builder
      end

      # Sets role code for the new UESURI.
      #
      # @param [String] role_code Role code
      #
      # @return [UESURIBuilder] Instance of UESURIBuilder
      def set_role_code(role_code)
        @role_code = role_code
        return self
      end

      # Sets territory ID for the new UESURI. In case artifact code
      # and artifact ID are not set, given territory ID is also used
      # in artifact UESURI part (forming valid territory UESURI).
      #
      # @param [String] territory_id Territory ID
      #
      # @return [UESURIBuilder] Instance of UESURIBuilder
      def self.set_territory_id(territory_id)
        builder = UESURIBuilder.new
        builder.set_territory_id(territory_id)
        return builder
      end

      # Sets territory ID for the new UESURI. In case artifact code
      # and artifact ID are not set, given territory ID is also used
      # in artifact UESURI part (forming valid territory UESURI).
      #
      # @param [String] territory_id Territory ID
      #
      # @return [UESURIBuilder] Instance of UESURIBuilder
      def set_territory_id(territory_id)
        @territory_id = territory_id
        return self
      end

      # Sets territory code for the new UESURI. In case artifact code
      # and artifact ID are not set, given territory code is also used
      # in artifact UESURI part (forming valid territory UESURI).
      #
      # @param [String] territory_code Territory code
      #
      # @return [UESURIBuilder] Instance of UESURIBuilder
      def self.set_territory_code(territory_code)
        builder = UESURIBuilder.new
        builder.set_territory_code(territory_code)
        return builder
      end

      # Sets territory code for the new UESURI. In case artifact code
      # and artifact ID are not set, given territory code is also used
      # in artifact UESURI part (forming valid territory UESURI).
      #
      # @param [String] territory_code Territory code
      #
      # @return [UESURIBuilder] Instance of UESURIBuilder
      def set_territory_code(territory_code)
        @territory_code = territory_code
        return self
      end

      # Sets artifact ID for the new UESURI. In case artifact code
      # and artifact ID are not set, territory UESURI part is used
      # as artifact UESURI part (forming valid territory UESURI).
      #
      # @param [String] artifact_id Artifact ID
      #
      # @return [UESURIBuilder] Instance of UESURIBuilder
      def self.set_artifact_id(artifact_id)
        builder = UESURIBuilder.new
        builder.set_artifact_id(artifact_id)
        return builder
      end

      # Sets artifact ID for the new UESURI. In case artifact code
      # and artifact ID are not set, territory UESURI part is used
      # as artifact UESURI part (forming valid territory UESURI).
      #
      # @param [String] artifact_id Artifact ID
      #
      # @return [UESURIBuilder] Instance of UESURIBuilder
      def set_artifact_id(artifact_id)
        @artifact_id = artifact_id
        return self
      end

      # Sets artifact code for the new UESURI. In case artifact code
      # and artifact ID are not set, territory UESURI part is used
      # as artifact UESURI part (forming valid territory UESURI).
      #
      # @param [String] artifact_code Artifact code
      #
      # @return [UESURIBuilder] Instance of UESURIBuilder
      def self.set_artifact_code(artifact_code)
        builder = UESURIBuilder.new
        builder.set_artifact_code(artifact_code)
        return builder
      end

      # Sets artifact code for the new UESURI. In case artifact code
      # and artifact ID are not set, territory UESURI part is used
      # as artifact UESURI part (forming valid territory UESURI).
      #
      # @param [String] artifact_code Artifact code
      #
      # @return [UESURIBuilder] Instance of UESURIBuilder
      def set_artifact_code(artifact_code)
        @artifact_code = artifact_code
        return self
      end

      # Sets object ID for the new UESURI.
      #
      # @param [String] object_id Object ID
      #
      # @return [UESURIBuilder] Instance of UESURIBuilder
      def self.set_object_id(object_id)
        builder = UESURIBuilder.new
        builder.set_object_id(object_id)
        return builder
      end

      # Sets object ID for the new UESURI.
      #
      # @param [String] object_id Object ID
      #
      # @return [UESURIBuilder] Instance of UESURIBuilder
      def set_object_id(object_id)
        @object_id = object_id
        return self
      end

      # Sets object code for the new UESURI.
      #
      # @param [String] object_code Object code
      #
      # @return [UESURIBuilder] Instance of UESURIBuilder
      def self.set_object_code(object_code)
        builder = UESURIBuilder.new
        builder.set_object_code(object_code)
        return builder
      end

      # Sets object code for the new UESURI.
      #
      # @param [String] object_code Object code
      #
      # @return [UESURIBuilder] Instance of UESURIBuilder
      def set_object_code(object_code)
        @object_code = object_code
        return self
      end

      # Sets use case ID for the new UESURI.
      #
      # @param [String] use_case_id Use case ID
      #
      # @return [UESURIBuilder] Instance of UESURIBuilder
      def self.set_use_case_id(use_case_id)
        builder = UESURIBuilder.new
        builder.set_use_case_id(use_case_id)
        return builder
      end

      # Sets use case ID for the new UESURI.
      #
      # @param [String] use_case_id Use case ID
      #
      # @return [UESURIBuilder] Instance of UESURIBuilder
      def set_use_case_id(use_case_id)
        @use_case_id = use_case_id
        return self
      end

      # Sets use case code for the new UESURI.
      #
      # @param [String] use_case_code Use case code
      #
      # @return [UESURIBuilder] Instance of UESURIBuilder
      def self.set_use_case_code(use_case_code)
        builder = UESURIBuilder.new
        builder.set_use_case_code(use_case_code)
        return builder
      end

      # Sets use case code for the new UESURI.
      #
      # @param [String] use_case_code Use case code
      #
      # @return [UESURIBuilder] Instance of UESURIBuilder
      def set_use_case_code(use_case_code)
        @use_case_code = use_case_code
        return self
      end

      # Sets use case action for the new UESURI.
      #
      # @param [String] action Use case action
      #
      # @return [UESURIBuilder] Instance of UESURIBuilder
      def self.set_action(action)
        builder = UESURIBuilder.new
        builder.set_action(action)
        return builder
      end

      # Sets use case action for the new UESURI.
      #
      # @param [String] action Use case action
      #
      # @return [UESURIBuilder] Instance of UESURIBuilder
      def set_action(action)
        @action = action
        return self
      end

      # Clears any existing UESURI parameters.
      #
      # @return [UESURIBuilder] Instance of UESURIBuilder
      def clear_parameters
        @parameters = nil
        return self
      end

      # Adds parameter for the new UESURI.
      #
      # @param [String] name Parameter name
      # @param [String] value Parameter value
      #
      # @return [UESURIBuilder] Instance of UESURIBuilder
      def self.add_parameter(name, value)
        builder = UESURIBuilder.new
        builder.add_parameter(name, value)
        return builder
      end

      # Adds parameter for the new UESURI.
      #
      # @param [String] name Parameter name
      # @param [String] value Parameter value
      #
      # @return [UESURIBuilder] Instance of UESURIBuilder
      def add_parameter(name, value)
        if !@parameters
          @parameters = []
        end
        @parameters << UU::OS::UESURI::Parameter.new(name, value)
        return self
      end

      # Sets fragment for the new UESURI.
      #
      # @param [String] fragment Fragment
      #
      # @return [UESURIBuilder] Instance of UESURIBuilder
      def self.set_fragment(fragment)
        builder = UESURIBuilder.new
        builder.set_fragment(fragment)
        return builder
      end

      # Sets fragment for the new UESURI.
      #
      # @param [String] fragment Fragment
      #
      # @return [UESURIBuilder] Instance of UESURIBuilder
      def set_fragment(fragment)
        @fragment = fragment
        return self
      end

      # Creates new instance of UESURI.
      #
      # @return [UU::OS::UESURI] Instance of UESURI
      def to_uesuri
        if (!@territory_id) && (!@territory_code)
          raise 'Territory must be set in order to create valid UESURI.'
        end
        uri_string = 'ues:'
        if (@role_code) || (@role_id)
          uri_string << url_encode(@role_code.to_s) if @role_code
          uri_string << "[#{@role_id.to_s}]" if @role_id
          uri_string << '@'
        end
        if (@territory_code) || (@territory_id)
          uri_string << url_encode(@territory_code.to_s) if @territory_code
          uri_string << "[#{@territory_id.to_s}]" if @territory_id
        end
        if (@artifact_code) || (@artifact_id)
          uri_string << ':'
          uri_string << url_encode(@artifact_code.to_s) if @artifact_code
          uri_string << "[#{@artifact_id.to_s}]" if @artifact_id
        else
          uri_string << ':'
          uri_string << url_encode(@territory_code.to_s) if @territory_code
          uri_string << "[#{@territory_id.to_s}]" if @territory_id
        end
        if (@object_code) || (@object_id)
          uri_string << ':'
          uri_string << url_encode(@object_code.to_s) if @object_code
          uri_string << "[#{@object_id.to_s}]" if @object_id
        end
        if (@use_case_code) || (@use_case_id) || (@action) || (@parameters)
          uri_string << '?'
          if (@use_case_code) || (@use_case_id)
            uri_string << url_encode(@use_case_code.to_s) if @use_case_code
            uri_string << "[#{@use_case_id.to_s}]" if @use_case_id
          end
          uri_string << ":#{@action}" if (!@action.nil?) && (!@action.empty?)
          if (@parameters)
            @parameters.each do |parameter|
              uri_string << "&#{parameter}"
            end
          end
        end
        if (@fragment)
          uri_string << "##{url_encode(@fragment.to_s)}"
        end
        UU::OS::UESURI.new(uri_string)
      end

      private

      # Encodes given value with URL-encoding.
      #
      # @param [String] value Value to be encoded
      # @return [String] Encoded value
      # @private
      def url_encode(value)
        UU::OS::UESURI.send(:url_encode, value)
      end

    end

  end
end
