require 'uu/os/qos/qos_handler'
require 'uu/os/rest'
require 'uu/os/uesuri'

module UU
  module OS
    
    # Use case log service.
    module UseCaseLog
      extend self
      
      # Service path
      PATH = 'ues/platform/appl/UESUseCaseLog';
      
      # Adds a new record to the use case log.
      # @param [UU::OS::UESURI] UESURI of a territory where the use case was executed. 
      # @param [UU::OS::UseCaseLog::UseCaseLogAddRecord] Attributes of a new use case log record.
      def add_record(territory_uri, log_record)
        svc = UU::OS::REST::RemoteClient.new(UseCaseLog)
        payload = UU::OS::UseCaseLog::UseCaseLogAddRecord.new(log_record).to_json();
        UU::OS::QoS::QoSHandler.auto_retry do
          svc.post('addRecord', territory_uri, payload)
        end
      end
      
      # Returns a list of use case log records of an artifact.
      # 
      # @param [String, UU::OS::UESURI] artifact_uri UESURI of an artifact, its use case log record should be returned.
      # @param [UU::OS::UseCaseLog::UseCaseLogGetRecordList] criteria Additional parameters of the use case log record, which should be returned.
      #
      # @return [UU::OS::REST::ResultList<UseCaseLog::UseCaseLogGetRecordList, UseCaseLog::UseCaseLogRecordEntry>] List of use case log records of an artifact.
      #
      # @example
      #   # Get record list for specified artifact without criteria.
      #   UU::OS::UseCaseLog.get_record_list('ues:UNI:TESTARTIFACT')
      #
      #   # Get record list with parameters.
      #   UU::OS::UseCaseLog.get_record_list('ues:UNI:TESTARTIFACT', :start_date=>'2017-01-01', :end_date=>'2017-01-10', :page_index=>0, :page_size=>10)
      def get_record_list(artifact_uri, criteria = nil)
        svc = UU::OS::REST::RemoteClient.new(UseCaseLog, 'uu/os/UseCaseLog')
        dto = UU::OS::UseCaseLog::UseCaseLogGetRecordList.new(criteria).to_json()
        
        UU::OS::QoS::QoSHandler.auto_retry do
          res = svc.post('getRecordList', artifact_uri, dto)
          return UU::OS::REST::ResultList.new(UseCaseLog::UseCaseLogGetRecordList, UseCaseLog::UseCaseLogRecordEntry, res)
        end
      end
    end
  end
end

# Require all from 'use case log'
Dir[File.dirname(__FILE__) + '/use_case_log/**/*.rb'].each {|file| require file.gsub('\\','/').split('/lib/').last[0..-4] }
